/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/

package com.piercefuller.a7trk.a7tview;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.HierarchyBoundsAdapter;
import java.awt.event.HierarchyBoundsListener;
import java.awt.event.HierarchyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.piercefuller.a7trk.decode.MultiSource;
import com.piercefuller.a7trk.decode.Record;
import com.piercefuller.a7trk.decode.Source;
import com.piercefuller.a7trk.decode.TapeDecoder;

@SuppressWarnings("serial")
public class A7tView extends JFrame {
    private MultiSource     multiSource  = null;
    // private Source source = null;
    private int             zoom         = 1;

    private ProgressView    progress     = null;
    private SourceControl[] sourceControl;
    private int             screenWidth;
    private int             screenHeight;
    private boolean         ignoreRecnum = false;

    private Colors          colors       = new Colors();

    private JButton         fileButton;
    private JFileChooser    openFileDialog1;
    private JButton         zoomInButton;
    private JButton         zoomOutButton;
    private ViewScreen      viewScreen;
    private JButton         configButton;
    private JCheckBox       rawCheckbox;
    private JCheckBox       lumpCheckbox;
    private JButton         zoomBackButton;
    private JCheckBox       dirCheckbox;
    private JSpinner        positionSpinner;
    private long            maximum1;
    private long            minimum1;
    private JCheckBox       centerCheckbox;
    private JCheckBox       zeroCheckbox;
    private JLabel          label1;
    private JLabel          label2;
    private JLabel          label3;
    private JLabel          label4;
    private JLabel          label5;
    private JLabel          label6;
    private JLabel          label7;
    private JCheckBox       clipCheckbox;
    private JButton         colorButton;
    private JLabel          zoomLabel;
    private JLabel          infoLabel;
    private JButton         evaluateButton;
    private JButton         nextErrButton;
    private JButton         calibrateButton;
    private JButton         recordEndButton;
    private JButton         writeButton;
    private JSpinner        recordSpinner;
    // private int maximum2;
    // private int minimum2;
    private JButton         recordBeginningButton;
    private JCheckBox       baseCheckbox;
    private JButton         markButton;
    private JCheckBox       crowdCheckbox;
    private JLabel          viewRangeLabel;
    private JButton         setStartButton;
    private JButton         setEndButton;
    private JButton         writeInfoButton;

    private class Runner implements Runnable {
        public static final int Calibrate = 0;
        public static final int Mark      = 1;
        public static final int Evaluate  = 2;
        public static final int Write     = 3;
        public static final int WriteInfo = 4;

        private int             whatToDo;

        public Runner(final int whatToDo) {
            this.whatToDo = whatToDo;
        }

        public void run() {
            try {
                switch (whatToDo) {
                case Calibrate:
                    multiSource.clone().calibrate(progress, multiSource);
                    break;
                case Mark:
                    multiSource.clone().mark(progress, multiSource);
                    break;
                case Evaluate:
                    multiSource.clone().evaluate(progress, multiSource);
                    break;
                case Write:
                    multiSource.clone().write(progress, multiSource);
                    break;
                case WriteInfo:
                    multiSource.clone().writeInfo();
                    break;
                }
            } catch (Exception e) {
            }
        }
    }

    // / <summary>
    // / Required designer variable.
    // / </summary>
    public A7tView() {
        super("A7tView");
        initialize();
    }

    public A7tView(final String filename) {
        super("A7tView " + filename);
        initialize();
        openData(filename);
    }

    private void initialize() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        fileButton = new JButton();
        openFileDialog1 = new JFileChooser();
        zoomInButton = new JButton();
        zoomOutButton = new JButton();
        configButton = new JButton();
        rawCheckbox = new JCheckBox();
        lumpCheckbox = new JCheckBox();
        zoomBackButton = new JButton();
        dirCheckbox = new JCheckBox();
        positionSpinner = new JSpinner();
        centerCheckbox = new JCheckBox();
        zeroCheckbox = new JCheckBox();
        label1 = new JLabel();
        label2 = new JLabel();
        label3 = new JLabel();
        label4 = new JLabel();
        label5 = new JLabel();
        label6 = new JLabel();
        label7 = new JLabel();
        clipCheckbox = new JCheckBox();
        colorButton = new JButton();
        zoomLabel = new JLabel();
        infoLabel = new JLabel();
        evaluateButton = new JButton();
        nextErrButton = new JButton();
        calibrateButton = new JButton();
        recordEndButton = new JButton();
        writeButton = new JButton();
        recordSpinner = new JSpinner();
        recordBeginningButton = new JButton();
        baseCheckbox = new JCheckBox();
        markButton = new JButton();
        crowdCheckbox = new JCheckBox();
        viewRangeLabel = new JLabel();
        setStartButton = new JButton();
        setEndButton = new JButton();
        writeInfoButton = new JButton();

        sourceControl = new SourceControl[5];
        sourceControl[0] = new SourceControl(rawCheckbox, colors.getRawColor(), 2048);
        sourceControl[1] = new SourceControl(lumpCheckbox, colors.getLumpColor(), 9000);
        sourceControl[2] = new SourceControl(baseCheckbox, colors.getBaseColor(), 9000);
        sourceControl[3] = new SourceControl(dirCheckbox, colors.getDirectionColor(), 9000);
        sourceControl[4] = new SourceControl(crowdCheckbox, colors.getCrowdColor(), 9000);

        Box top = new Box(BoxLayout.X_AXIS);
        Box bot = new Box(BoxLayout.X_AXIS);

        //
        // fileButton
        //
        // fileButton.setLocation(new Point(-8, 0));
        // fileButton.setName("fileButton");
        // fileButton.setSize(new Dimension(40, 24));
        // fileButton.setTabIndex(0);
        fileButton.setText("File");
        fileButton.addActionListener(fileButtonClick);
        //
        // openFileDialog1
        //
        // openFileDialog1.addChoosableFileFilter(new FileNameExtensionFilter("xml"));
        // openFileDialog1.addChoosableFileFilter(new FileNameExtensionFilter("dat"));
        //
        // zoomInButton
        //
        // zoomInButton.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // zoomInButton.setLocation(new Point(712, 0));
        // zoomInButton.setName("zoomInButton");
        // zoomInButton.setSize(new Dimension(24, 23));
        // zoomInButton.setTabIndex(4);
        zoomInButton.setText("+");
        zoomInButton.addActionListener(zoomInButtonClick);
        //
        // zoomOutButton
        //
        // zoomOutButton.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // zoomOutButton.setLocation(new Point(736, 0));
        // zoomOutButton.setName("zoomOutButton");
        // zoomOutButton.setSize(new Dimension(24, 23));
        // zoomOutButton.setTabIndex(5);
        zoomOutButton.setText("-");
        zoomOutButton.addActionListener(zoomOutButtonClick);
        //
        // configButton
        //
        // configButton.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // configButton.setLocation(new Point(600, 0));
        // configButton.setName("configButton");
        // configButton.setSize(new Dimension(40, 23));
        // configButton.setTabIndex(9);
        configButton.setText("Conf");
        configButton.addActionListener(configButtonClick);
        //
        // rawCheckbox
        //
        rawCheckbox.setSelected(true);
        // rawCheckbox.setLocation(new Point(40, 0));
        // rawCheckbox.setName("rawCheckbox");
        // rawCheckbox.setSize(new Dimension(56, 24));
        // rawCheckbox.setTabIndex(10);
        rawCheckbox.setText("Raw");
        rawCheckbox.addActionListener(somethingChanged);
        //
        // lumpCheckbox
        //
        lumpCheckbox.setSelected(true);
        // lumpCheckbox.setLocation(new Point(88, 0));
        // lumpCheckbox.setName("lumpCheckbox");
        // lumpCheckbox.setSize(new Dimension(56, 24));
        // lumpCheckbox.setTabIndex(11);
        lumpCheckbox.setText("Lump");
        lumpCheckbox.addActionListener(somethingChanged);
        //
        // zoomBackButton
        //
        // zoomBackButton.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // zoomBackButton.setLocation(new Point(680, 0));
        // zoomBackButton.setName("zoomBackButton");
        // zoomBackButton.setSize(new Dimension(32, 23));
        // zoomBackButton.setTabIndex(0);
        zoomBackButton.setText("1");
        zoomBackButton.addActionListener(zoomBackButtonClick);
        //
        // dirCheckbox
        //
        dirCheckbox.setSelected(true);
        // dirCheckbox.setLocation(new Point(184, 0));
        // dirCheckbox.setName("dirCheckbox");
        // dirCheckbox.setSize(new Dimension(40, 24));
        // dirCheckbox.setTabIndex(13);
        dirCheckbox.setText("Dir");
        dirCheckbox.addActionListener(somethingChanged);
        //
        // positionSpinner
        //
        // positionSpinner.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        // positionSpinner.setLocation(new Point(208, 424));
        // positionSpinner.setName("positionSpinner");
        positionSpinner.setSize(new Dimension(88, 20));
        // positionSpinner.setTabIndex(15);
        positionSpinner.addChangeListener(positionSpinnerValueChanged);
        //
        // centerCheckbox
        //
        centerCheckbox.setSelected(true);
        // centerCheckbox.setLocation(new Point(264, 0));
        // centerCheckbox.setName("centerCheckbox");
        // centerCheckbox.setSize(new Dimension(64, 24));
        // centerCheckbox.setTabIndex(16);
        centerCheckbox.setText("Center");
        centerCheckbox.addActionListener(somethingChanged);
        //
        // zeroCheckbox
        //
        zeroCheckbox.setSelected(true);
        // zeroCheckbox.setLocation(new Point(320, 0));
        // zeroCheckbox.setName("zeroCheckbox");
        // zeroCheckbox.setSize(new Dimension(48, 24));
        // zeroCheckbox.setTabIndex(17);
        zeroCheckbox.setText("Zero");
        zeroCheckbox.addActionListener(somethingChanged);
        //
        // label1
        //
        label1.setLocation(new Point(8, 56));
        label1.setName("label1");
        label1.setSize(new Dimension(16, 16));
        // label1.setTabIndex(18);
        label1.setText("P");
        //
        // label2
        //
        label2.setLocation(new Point(8, 104));
        label2.setName("label2");
        label2.setSize(new Dimension(16, 16));
        // label2.setTabIndex(19);
        label2.setText("5");
        //
        // label3
        //
        label3.setLocation(new Point(8, 152));
        label3.setName("label3");
        label3.setSize(new Dimension(16, 16));
        // label3.setTabIndex(20);
        label3.setText("4");
        //
        // label4
        //
        label4.setLocation(new Point(8, 200));
        label4.setName("label4");
        label4.setSize(new Dimension(16, 16));
        // label4.setTabIndex(21);
        label4.setText("3");
        //
        // label5
        //
        label5.setLocation(new Point(8, 256));
        label5.setName("label5");
        label5.setSize(new Dimension(16, 16));
        // label5.setTabIndex(22);
        label5.setText("2");
        //
        // label6
        //
        label6.setLocation(new Point(8, 304));
        label6.setName("label6");
        label6.setSize(new Dimension(16, 16));
        // label6.setTabIndex(23);
        label6.setText("1");
        //
        // label7
        //
        label7.setLocation(new Point(8, 352));
        label7.setName("label7");
        label7.setSize(new Dimension(16, 16));
        // label7.setTabIndex(24);
        label7.setText("0");
        //
        // clipCheckbox
        //
        clipCheckbox.setSelected(true);
        // clipCheckbox.setLocation(new Point(368, 0));
        // clipCheckbox.setName("clipCheckbox");
        // clipCheckbox.setSize(new Dimension(48, 24));
        // clipCheckbox.setTabIndex(25);
        clipCheckbox.setText("Clip");
        clipCheckbox.addActionListener(somethingChanged);
        //
        // colorButton
        //
        // colorButton.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // colorButton.setLocation(new Point(640, 0));
        // colorButton.setName("colorButton");
        // colorButton.setSize(new Dimension(40, 23));
        // colorButton.setTabIndex(26);
        colorButton.setText("Color");
        colorButton.addActionListener(colorButtonClick);
        //
        // zoomLabel
        //
        // zoomLabel.setAnchor((JAnchorStyles.top | JAnchorStyles.right));
        // zoomLabel.setLocation(new Point(768, 0));
        // zoomLabel.setName("zoomLabel");
        // zoomLabel.setSize(new Dimension(32, 24));
        // zoomLabel.setTabIndex(27);
        zoomLabel.setText("zoom");
        // zoomLabel.setTextAlign(ContentAlignment.middleCenter);
        //
        // infoLabel
        //
        // infoLabel.setAnchor((JAnchorStyles.bottom | JAnchorStyles.right));
        // infoLabel.setLocation(new Point(512, 424));
        // infoLabel.setName("infoLabel");
        // infoLabel.setSize(new Dimension(280, 16));
        // infoLabel.setTabIndex(29);
        // infoLabel.setTextAlign(ContentAlignment.middleRight);
        //
        // evaluateButton
        //
        // evaluateButton.setLocation(new Point(480, 0));
        // evaluateButton.setName("evaluateButton");
        // evaluateButton.setSize(new Dimension(40, 23));
        // evaluateButton.setTabIndex(30);
        evaluateButton.setText("Evaluate");
        evaluateButton.addActionListener(evaluateButtonClick);
        //
        // nextErrButton
        //
        // nextErrButton.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        nextErrButton.setEnabled(false);
        // nextErrButton.setLocation(new Point(384, 424));
        // nextErrButton.setName("nextErrButton");
        // nextErrButton.setSize(new Dimension(56, 23));
        // nextErrButton.setTabIndex(32);
        nextErrButton.setText("Next Err");
        nextErrButton.addActionListener(nextErrButtonClick);
        //
        // calibrateButton
        //
        // calibrateButton.setLocation(new Point(408, 0));
        // calibrateButton.setName("calibrateButton");
        // calibrateButton.setSize(new Dimension(32, 23));
        // calibrateButton.setTabIndex(33);
        calibrateButton.setText("Cal");
        calibrateButton.addActionListener(calibrateButtonClick);
        //
        // recordEndButton
        //
        // recordEndButton.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        recordEndButton.setEnabled(false);
        // recordEndButton.setLocation(new Point(344, 424));
        // recordEndButton.setName("recordEndButton");
        // recordEndButton.setSize(new Dimension(40, 23));
        // recordEndButton.setTabIndex(35);
        recordEndButton.setText("End");
        recordEndButton.addActionListener(recordEndButtonClick);
        //
        // writeButton
        //
        // writeButton.setLocation(new Point(520, 0));
        // writeButton.setName("writeButton");
        // writeButton.setSize(new Dimension(40, 23));
        // writeButton.setTabIndex(36);
        writeButton.setText("Write");
        writeButton.addActionListener(writeButtonClick);
        //
        // recordSpinner
        //
        // recordSpinner.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        recordSpinner.setEnabled(false);
        // recordSpinner.setLocation(new Point(440, 424));
        // recordSpinner.setName("recordSpinner");
        // recordSpinner.setSize(new Dimension(64, 20));
        // recordSpinner.setTabIndex(37);
        // maximum2 = 1;
        // minimum2 = 1;
        recordSpinner.getModel().setValue(new Integer(1));
        recordSpinner.addChangeListener(recordSpinnerValueChanged);
        //
        // recordBeginningButton
        //
        // recordBeginningButton.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        recordBeginningButton.setEnabled(false);
        // recordBeginningButton.setLocation(new Point(296, 424));
        // recordBeginningButton.setName("recordBeginningButton");
        // recordBeginningButton.setSize(new Dimension(48, 23));
        // recordBeginningButton.setTabIndex(31);
        recordBeginningButton.setText("Begin");
        recordBeginningButton.addActionListener(recordBeginningButtonClick);
        //
        // baseCheckbox
        //
        baseCheckbox.setSelected(true);
        // baseCheckbox.setCheckState(JCheckState.isSelected());
        // baseCheckbox.setLocation(new Point(136, 0));
        // baseCheckbox.setName("baseCheckbox");
        // baseCheckbox.setSize(new Dimension(56, 24));
        // baseCheckbox.setTabIndex(38);
        baseCheckbox.setText("Base");
        baseCheckbox.addActionListener(somethingChanged);
        //
        // markButton
        //
        // markButton.setLocation(new Point(440, 0));
        // markButton.setName("markButton");
        // markButton.setSize(new Dimension(40, 23));
        // markButton.setTabIndex(39);
        markButton.setText("Mark");
        markButton.addActionListener(markButtonClick);
        //
        // crowdCheckbox
        //
        crowdCheckbox.setSelected(true);
        // crowdCheckbox.setLocation(new Point(224, 0));
        // crowdCheckbox.setName("crowdCheckbox");
        // crowdCheckbox.setSize(new Dimension(40, 24));
        // crowdCheckbox.setTabIndex(40);
        crowdCheckbox.setText("Cr");
        crowdCheckbox.addActionListener(somethingChanged);
        //
        // viewRangeLabel
        //
        // viewRangeLabel.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        // viewRangeLabel.setLocation(new Point(8, 424));
        // viewRangeLabel.setName("viewRangeLabel");
        // viewRangeLabel.setSize(new Dimension(144, 16));
        // viewRangeLabel.setTabIndex(41);
        //
        // setStartButton
        //
        // setStartButton.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        // setStartButton.setLocation(new Point(160, 424));
        // setStartButton.setName("setStartButton");
        // setStartButton.setSize(new Dimension(24, 23));
        // setStartButton.setTabIndex(42);
        setStartButton.setText("S");
        setStartButton.addActionListener(setStartButtonClick);
        //
        // setEndButton
        //
        // setEndButton.setAnchor((JAnchorStyles.bottom | JAnchorStyles.left));
        // setEndButton.setLocation(new Point(184, 424));
        // setEndButton.setName("setEndButton");
        // setEndButton.setSize(new Dimension(24, 23));
        // setEndButton.setTabIndex(43);
        setEndButton.setText("E");
        setEndButton.addActionListener(setEndButtonClick);
        //
        // writeInfoButton
        //
        // writeInfoButton.setLocation(new Point(560, 0));
        // writeInfoButton.setName("writeInfoButton");
        // writeInfoButton.setSize(new Dimension(32, 23));
        // writeInfoButton.setTabIndex(44);
        writeInfoButton.setText("WX");
        writeInfoButton.addActionListener(writeInfoButtonClick);

        //
        // A7tView
        //

        top.add(fileButton);
        top.add(rawCheckbox);
        top.add(lumpCheckbox);
        top.add(baseCheckbox);
        top.add(dirCheckbox);
        top.add(crowdCheckbox);
        top.add(centerCheckbox);
        top.add(zeroCheckbox);
        top.add(clipCheckbox);
        top.add(Box.createGlue());
        top.add(calibrateButton);
        top.add(markButton);
        top.add(evaluateButton);
        top.add(writeButton);
        top.add(writeInfoButton);
        top.add(Box.createGlue());
        top.add(configButton);
        top.add(colorButton);
        top.add(Box.createGlue());
        top.add(zoomLabel);
        top.add(Box.createHorizontalStrut(6));
        top.add(zoomInButton);
        top.add(zoomOutButton);
        top.add(zoomBackButton);

        bot.add(Box.createGlue());
        bot.add(setStartButton);
        bot.add(setEndButton);
        bot.add(Box.createHorizontalStrut(6));
        bot.add(positionSpinner);
        bot.add(Box.createGlue());
        bot.add(viewRangeLabel);
        bot.add(Box.createGlue());
        bot.add(Box.createGlue());
        bot.add(recordSpinner);
        bot.add(recordBeginningButton);
        bot.add(recordEndButton);
        bot.add(nextErrButton);
        bot.add(Box.createGlue());
        bot.add(infoLabel);
        bot.add(Box.createGlue());

        //
        // viewScreen
        //
        viewScreen = new ViewScreen(this, sourceControl, colors);
        // viewScreen.setAnchor((((JAnchorStyles.top | JAnchorStyles.bottom) | JAnchorStyles.left) |
        // JAnchorStyles.right));
        // viewScreen.setLocation(new Point(24, 24));
        // viewScreen.setName("ViewScreen");
        // viewScreen.setSize(new Dimension(768, 400));
        // viewScreen.setTabIndex(8);
        viewScreen.addHierarchyBoundsListener(viewScreenResize);
        viewScreen.addMouseListener(viewScreenMouseDown);

        top.validate();
        top.setMaximumSize(new Dimension(10000, top.getPreferredSize().height));
        bot.validate();
        bot.setMaximumSize(new Dimension(10000, bot.getPreferredSize().height));
        getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
        add(top);
        add(viewScreen);
        add(bot);

        Dimension dim = new Dimension(Math.max(top.getPreferredSize().width, bot.getPreferredSize().width), 600);
        setMinimumSize(dim);
        setPreferredSize(dim);
        // add(label7);
        // add(label6);
        // add(label5);
        // add(label4);
        // add(label3);
        // add(label2);
        // add(label1);
        setName("A7tView");
        setTitle("A7tView");

        pack();
        setVisible(true);
    }

    public MultiSource getMultiSource() {
        return multiSource;
    }

    public int getRawZoom() {
        return sourceControl[0].getZoom();
    }

    public void setRawZoom(final int i) {
        sourceControl[0].setZoom(i);
    }

    public int getLumpZoom() {
        return sourceControl[1].getZoom();
    }

    public void setLumpZoom(final int i) {
        sourceControl[1].setZoom(i);
        sourceControl[2].setZoom(i);
        sourceControl[3].setZoom(i);
    }

    public Source getSource() {
        if (multiSource == null) {
            return null;
        }
        return multiSource.getSource();
    }

    public int getZoom() {
        return zoom;
    }

    public long getZoomWidth(final int width) {
        return zoom * width / 2;
    }

    public long getZoomWidth() {
        return getZoomWidth(screenWidth);
    }

    public boolean centerEnabled() {
        return centerCheckbox.isSelected();
    }

    public boolean zeroEnabled() {
        return zeroCheckbox.isSelected();
    }

    public boolean clipEnabled() {
        return clipCheckbox.isSelected();
    }

    protected void setPosition(long pos) {
        if (pos > maximum1) {
            pos = maximum1;
        }
        if (pos < minimum1) {
            pos = minimum1;
        }
        if (((Number)positionSpinner.getModel().getValue()).longValue() != pos) {
            positionSpinner.setValue(new Long(pos));
        } else {
            needUpdate();
        }
    }

    public long getPosition() {
        return ((Number)positionSpinner.getModel().getValue()).longValue();
    }

    public void needUpdate() {
        sourceControl[0].setColor(colors.getRawColor());
        sourceControl[1].setColor(colors.getLumpColor());
        sourceControl[2].setColor(colors.getBaseColor());
        sourceControl[3].setColor(colors.getDirectionColor());
        sourceControl[4].setColor(colors.getCrowdColor());

        zoomLabel.setText(Integer.toString(zoom));

        screenWidth = viewScreen.getWidth() - 2 * viewScreen.SideBorder;
        screenHeight = viewScreen.getHeight();
        int nchannel = 0;
        int channelHeight = 0;

        if (multiSource != null) {
            viewRangeLabel.setText(multiSource.getStart() + " - " + multiSource.getEnd());
            multiSource.setPosition(getPosition());
            nchannel = multiSource.getSource().getNumChannels();
            channelHeight = (screenHeight - 20 - (viewScreen.StatusHeight + viewScreen.SliderHeight)) / nchannel;

            nextErrButton.setEnabled(multiSource.getErrorTotal() > 0);

            String st = "";
            if (multiSource.recordOriented()) {
                Record rec = multiSource.getRecord(getPosition());
                if (rec != null) {
                    // minimum2 = 1;
                    // maximum2 = multiSource.getNumRecords();
                    if (((Number)recordSpinner.getModel().getValue()).intValue() != rec.getNumber()) {
                        ignoreRecnum = true;
                        recordSpinner.setValue(new Integer(rec.getNumber()));
                    }
                    recordSpinner.setEnabled(true);
                    recordBeginningButton.setEnabled(true);
                    recordEndButton.setEnabled(true);
                    long relpos = getPosition() - rec.getStart();
                    st = "pos " + relpos + " cnt " + rec.getCount() + " sp " + ((TapeDecoder)multiSource).getSpacing() + ", " + rec.getErrors() + " err, "
                            + multiSource.getErrorTotal() + " Err " + multiSource.getNumRecords() + " Rec";
                } else {
                    int n = multiSource.getNumRecords();
                    if (n > 0) {
                        // minimum2 = 1;
                        // maximum2 = multiSource.getNumRecords();
                        recordSpinner.setEnabled(true);
                        recordBeginningButton.setEnabled(true);
                        recordEndButton.setEnabled(true);
                        st = multiSource.getErrorTotal() + " Total errors  " + n + " Records";
                    } else {
                        recordSpinner.setEnabled(false);
                        recordBeginningButton.setEnabled(false);
                        recordEndButton.setEnabled(false);
                    }
                }
            } else {
                recordSpinner.setEnabled(false);
                st = multiSource.getErrorTotal() + " Total errors";
            }
            infoLabel.setText(st);

            int labelOffset = 24 - 8 + 10 + channelHeight / 2;
            label1.setLocation(new Point(8, labelOffset + channelHeight * (0)));
            label2.setLocation(new Point(8, labelOffset + channelHeight * (1)));
            label3.setLocation(new Point(8, labelOffset + channelHeight * (2)));
            label4.setLocation(new Point(8, labelOffset + channelHeight * (3)));
            label5.setLocation(new Point(8, labelOffset + channelHeight * (4)));
            label6.setLocation(new Point(8, labelOffset + channelHeight * (5)));
            label7.setLocation(new Point(8, labelOffset + channelHeight * (6)));

            // long spos = getPosition() - getZoomWidth()/2;
            // long epos = spos + getZoomWidth();

            int nlevel = 1;
            Source s = multiSource.getSource().getSource();
            for (; s != null; s = s.getSource(), nlevel++) {
                ;
            }
            for (SourceControl element: sourceControl) {
                element.getCheckBox().setEnabled(false);
            }
            s = multiSource.getSource();
            for (int level = 0; level < nlevel; s = s.getSource(), level++) {
                for (SourceControl element: sourceControl) {
                    if (element.getCheckBox().getText().equals(s.getName())) {
                        element.getCheckBox().setEnabled(true);
                        break;
                    }
                }
            }
        }

        repaint();
    }

    private void openData(final String filename) {
        closeData();
        if (filename != null) {
            try {
                multiSource = new TapeDecoder(filename);
                setTitle("A7tView - " + filename);
                minimum1 = 0;
                maximum1 = multiSource.getLength();
            } catch (Exception ex) {
                infoLabel.setText(ex.toString());
                multiSource = null;
                minimum1 = 0;
                maximum1 = 0;
                positionSpinner.setValue(new Integer(0));
                ex.printStackTrace();
            }
        } else {
            setTitle("A7tView");
        }
        zoom = 1;
        setPosition(0);
    }

    private void closeData() {
        if (multiSource != null) {
            multiSource.close();
            multiSource = null;
        }
    }

    private Action                  fileButtonClick             = new AbstractAction("fileButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        closeData();
                                                                        openFileDialog1.showOpenDialog(A7tView.this);
                                                                        if (openFileDialog1.getSelectedFile() != null) {
                                                                            openData(openFileDialog1.getSelectedFile().getPath());
                                                                        }
                                                                    }
                                                                };

    private Action                  zoomInButtonClick           = new AbstractAction("zoomInButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (zoom > 1) {
                                                                            zoom /= 2;
                                                                        }
                                                                        needUpdate();
                                                                    }
                                                                };

    private Action                  zoomOutButtonClick          = new AbstractAction("zoomOutButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            if (2 * getZoomWidth() <= multiSource.getLength()) {
                                                                                zoom *= 2;
                                                                            }
                                                                        }
                                                                        needUpdate();
                                                                    }
                                                                };

    private MouseListener           viewScreenMouseDown         = new MouseAdapter() {
                                                                    @Override
                                                                    public void mouseClicked(final MouseEvent e) {
                                                                        int x = e.getPoint().x;
                                                                        int y = e.getPoint().y;
                                                                        if (y < screenHeight - (viewScreen.StatusHeight + viewScreen.SliderHeight)) {
                                                                            setPosition(getPosition()
                                                                                    + getZoomWidth((x - viewScreen.SideBorder - screenWidth / 2)));
                                                                        } else {
                                                                            if (y < screenHeight - viewScreen.SliderHeight) {
                                                                                setPosition(getPosition()
                                                                                        + getZoomWidth((x - viewScreen.SideBorder - screenWidth / 2)));
                                                                            } else {
                                                                                setPosition(multiSource.getLength() * (x - viewScreen.SideBorder) / screenWidth);
                                                                            }
                                                                        }
                                                                    }
                                                                };

    private HierarchyBoundsListener viewScreenResize            = new HierarchyBoundsAdapter() {
                                                                    @Override
                                                                    public void ancestorResized(final HierarchyEvent e) {
                                                                        needUpdate();
                                                                    }
                                                                };

    private Action                  configButtonClick           = new AbstractAction("configButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            ConfigDialog c = new ConfigDialog(A7tView.this);
                                                                            c.setVisible(true);
                                                                            needUpdate();
                                                                        }
                                                                    }
                                                                };

    private Action                  zoomBackButtonClick         = new AbstractAction("zoomBackButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        zoom = 1;
                                                                        needUpdate();
                                                                    }
                                                                };

    private Action                  somethingChanged            = new AbstractAction("somethingChanged") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        needUpdate();
                                                                    }
                                                                };

    private ChangeListener          positionSpinnerValueChanged = new ChangeListener() {
                                                                    public void stateChanged(final ChangeEvent e) {
                                                                        setPosition(((Number)positionSpinner.getModel().getValue()).longValue());
                                                                    }
                                                                };

    private Action                  colorButtonClick            = new AbstractAction("colorButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        ColorDialog c = new ColorDialog(A7tView.this, colors);
                                                                        c.setVisible(true);
                                                                    }
                                                                };

    private Action                  evaluateButtonClick         = new AbstractAction("evaluateButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            if (progress != null) {
                                                                                progress.closeAll();
                                                                            }
                                                                            progress = new ProgressView("Evaluating...");
                                                                            progress.setVisible(true);
                                                                            new Thread(new Runner(Runner.Evaluate)).start();
                                                                        }
                                                                    }
                                                                };

    private Action                  recordBeginningButtonClick  = new AbstractAction("recordBeginningButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null && multiSource.recordOriented()) {
                                                                            long position = getPosition();
                                                                            Record rec = multiSource.getRecord(position);
                                                                            if (rec == null) {
                                                                                rec = multiSource.getPreviousRecord(position);
                                                                            }
                                                                            if (rec == null) {
                                                                                rec = multiSource.getNextRecord(position);
                                                                            }
                                                                            if (rec != null) {
                                                                                long newpos = rec.getStart();
                                                                                if (newpos != position) {
                                                                                    setPosition(newpos);
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                };

    private Action                  nextErrButtonClick          = new AbstractAction("nextErrButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null && multiSource.recordOriented()) {
                                                                            multiSource.setPosition(getPosition());
                                                                            setPosition(multiSource.getNextError());
                                                                        }
                                                                    }
                                                                };

    private Action                  calibrateButtonClick        = new AbstractAction("calibrateButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            if (progress != null) {
                                                                                progress.closeAll();
                                                                            }
                                                                            progress = new ProgressView("Calibrating...");
                                                                            progress.setVisible(true);
                                                                            new Thread(new Runner(Runner.Calibrate)).start();
                                                                        }
                                                                    }
                                                                };

    private Action                  recordEndButtonClick        = new AbstractAction("recordEndButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null && multiSource.recordOriented()) {
                                                                            long position = getPosition();
                                                                            Record rec = multiSource.getRecord(position);
                                                                            if (rec != null) {
                                                                                long newpos = rec.getEnd();
                                                                                if (newpos != position) {
                                                                                    setPosition(newpos);
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                };

    private Action                  writeButtonClick            = new AbstractAction("writeButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            if (progress != null) {
                                                                                progress.closeAll();
                                                                            }
                                                                            progress = new ProgressView("Writing...");
                                                                            progress.setVisible(true);
                                                                            new Thread(new Runner(Runner.Write)).start();
                                                                        }
                                                                    }
                                                                };

    private ChangeListener          recordSpinnerValueChanged   = new ChangeListener() {
                                                                    public void stateChanged(final ChangeEvent e) {
                                                                        if (ignoreRecnum) {
                                                                            ignoreRecnum = false;
                                                                            return;
                                                                        }
                                                                        int i = ((Number)recordSpinner.getModel().getValue()).intValue();
                                                                        if (multiSource != null) {
                                                                            Record rec = multiSource.getRecord(i);
                                                                            if (rec != null) {
                                                                                setPosition(rec.getStart());
                                                                            }
                                                                        }
                                                                    }
                                                                };

    private Action                  markButtonClick             = new AbstractAction("markButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            if (progress != null) {
                                                                                progress.closeAll();
                                                                            }
                                                                            progress = new ProgressView("Marking...");
                                                                            progress.setVisible(true);
                                                                            new Thread(new Runner(Runner.Mark)).start();
                                                                        }
                                                                    }
                                                                };

    private Action                  setStartButtonClick         = new AbstractAction("setStartButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            multiSource.setStart(getPosition());
                                                                        }
                                                                        needUpdate();
                                                                    }
                                                                };

    private Action                  setEndButtonClick           = new AbstractAction("setEndButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            multiSource.setEnd(getPosition());
                                                                        }
                                                                        needUpdate();
                                                                    }
                                                                };

    private Action                  writeInfoButtonClick        = new AbstractAction("writeInfoButtonClick") {
                                                                    public void actionPerformed(final ActionEvent e) {
                                                                        if (multiSource != null) {
                                                                            new Runner(Runner.WriteInfo).run();
                                                                        }
                                                                    }
                                                                };

    public static void main(final String[] args) {
        try {
            if (args.length > 0) {
                new A7tView(args[0]);
            } else {
                new A7tView();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
