/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.a7tview;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.piercefuller.a7trk.decode.Plottable;


/// <summary>
/// Summary description for PlotView.
/// </summary>
public class PlotView extends JFrame
{
	private Plottable plot;
	private Color[] colors = new Color[3];
	private JCheckBox[] legend = new JCheckBox[6];

	private JPanel panel1;
	private JLabel xMinLabel;
	private JLabel xMaxLabel;
	private JLabel yMinLabel;
	private JLabel yMaxLabel;

	public PlotView(Plottable p)
	{
		plot = p;
		
		Box top = new Box(BoxLayout.X_AXIS);
		Box left = new Box(BoxLayout.Y_AXIS);
		Box right = new Box(BoxLayout.Y_AXIS);
		Box center = new Box(BoxLayout.Y_AXIS);
		Box plotbot = new Box(BoxLayout.X_AXIS);
		
		panel1 = new JPanel();
		xMinLabel = new JLabel();
		xMaxLabel = new JLabel();
		yMinLabel = new JLabel();
		yMaxLabel = new JLabel();
		//panel1.setVisible(true);
		panel1.setPreferredSize(new Dimension(344, 200));
		//panel1.setMinimumSize(new Dimension(344, 200));
		//panel1.validate();

		top.add(Box.createGlue());
		top.add(new JLabel(plot.getName()));
		top.add(Box.createGlue());
		top.validate();

		plotbot.add(xMinLabel);
		plotbot.add(Box.createGlue());
		plotbot.add(new JLabel(plot.getYLabel()));
		plotbot.add(Box.createGlue());
		plotbot.add(xMaxLabel);
		plotbot.setMaximumSize(new Dimension(10000, plotbot.getPreferredSize().height));
		plotbot.validate();

		left.add(Box.createVerticalStrut(top.getPreferredSize().height));
		left.add(yMaxLabel);
		left.add(Box.createGlue());
		left.add(new JLabel(plot.getXLabel()));
		left.add(Box.createGlue());
		left.add(yMinLabel);
		left.add(Box.createVerticalStrut(plotbot.getPreferredSize().height));

		center.add(panel1);
		center.add(plotbot);

		colors[0] = Color.black;
		colors[1] = Color.blue;
		colors[2] = Color.red;

		setTitle(plot.getName());

		right.add(Box.createGlue());
		legend = new JCheckBox[plot.getNumDatasets()];
		for (int i = 0; i < legend.length; i++) 
		{
			legend[i] = new JCheckBox();
			legend[i].addActionListener(checkBox_CheckedChanged);
			legend[i].setText(plot.getDatasetName(i));
			legend[i].setForeground(colors[i % colors.length]);
			legend[i].setSelected(true);
			legend[i].setEnabled(true);
			legend[i].setVisible(true);
			right.add(legend[i]);
		}
		right.add(Box.createGlue());
		right.add(Box.createVerticalStrut(xMinLabel.getHeight()));

		getContentPane().setLayout(new BorderLayout());
		
		add(top, BorderLayout.NORTH);
		add(left, BorderLayout.WEST);
		add(center, BorderLayout.CENTER);
		add(right, BorderLayout.EAST);
		
		validate();
		panel1.setSize(new Dimension(344, 200));
		pack();
		repaint();
	}


	public void paint(Graphics g)
	{
		super.paint(g);

		g = panel1.getGraphics();
		int n = plot.getNumDatasets();
		if (n > legend.length) 
		{
			n = legend.length;
		}
		int[][] ydata = new int[n][];
		int[] xdata = plot.getDataset(0);
		int[] xPoints = new int[xdata.length];
		int[] yPoints = new int[xdata.length];
		for (int i = 0; i < n; i++) 
		{
			if (legend[i].isSelected()) 
			{
				ydata[i] = plot.getDataset(i+1);
			}
		}

		double minX = 1000000000;
		double maxX = -1000000000;
		double minY = 1000000000;
		double maxY = -1000000000;
		for (int i = 0; i < n; i++) 
		{
			if (legend[i].isSelected()) 
			{
				for (int j = 0; j < xdata.length; j++)
				{
					double x = xdata[j];
					double y = ydata[i][j];
					if (x > maxX) 
					{
						maxX = x;
					}
					if (x < minX) 
					{
						minX = x;
					}
					if (y > maxY) 
					{
						maxY = y;
					}
					if (y < minY) 
					{
						minY = y;
					}
				}
			}
		}
		xMinLabel.setText(Double.toString(minX));
		xMaxLabel.setText(Double.toString(maxX));
		yMinLabel.setText(Double.toString(minY));
		yMaxLabel.setText(Double.toString(maxY));

		g.setColor(Color.white);
		g.fillRect(panel1.getX(), panel1.getY(), panel1.getWidth(), panel1.getHeight());

		double XScale = 1.0;
		if (plot.isXLog()) 
		{
			double v = maxX - minX;
			if (v > 0) 
			{
				v = Math.log(v);
				XScale = (double)panel1.getWidth() / v;
			}
		} 
		else if (maxX > minX)
		{
			XScale = (double)panel1.getWidth() / (maxX - minX);
		}

		double YScale = 1.0;
		if (plot.isYLog()) 
		{
			double v = maxY - minY;
			if (v > 0) 
			{
				v = Math.log(v);
				YScale = (double)panel1.getHeight() / v;
			}
		} 
		else if (maxY > minY)
		{
			YScale = (double)panel1.getHeight() / (maxY - minY);
		}

		for (int i = 0; i < n; i++) 
		{
			if (legend[i].isSelected()) 
			{
				for (int j = 0; j < xdata.length; j++)
				{
					double x = (double)xdata[j] - minX;
					if (x > 0 && plot.isXLog()) 
					{
						x = Math.log(x);
					}
					double y = (double)ydata[i][j] - minY;
					if (y > 0 && plot.isYLog()) 
					{
						y = Math.log(y);
					}
					x = XScale*x;
					y = YScale*y;
					xPoints[j] = panel1.getX() + (int)x;
					yPoints[j] = panel1.getY() + panel1.getHeight() - (int)y;
				}
				g.setColor(colors[i % colors.length]);
				g.drawPolyline(xPoints, yPoints, xdata.length);
			}
		}
	}

	private Action checkBox_CheckedChanged = new AbstractAction("checkBox_CheckedChanged") { public void actionPerformed(ActionEvent e)
	{
		repaint();
	}};
}
