/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.a7tview;

import java.awt.event.*;
import javax.swing.*;
import com.piercefuller.a7trk.decode.*;


/// <summary>
/// Summary description for ProgressView.
/// </summary>
public class ProgressView extends JFrame implements ProgressWatcher
{
	private boolean proceed = true;
	private boolean finished = false;
	private Plottable myplot = null;
	private PlotView myplotview = null;

	private JProgressBar progressBar1;
	private JLabel label1;
	private JButton button1;

	public ProgressView(String title)
	{		
		button1 = new JButton();
		button1.setAlignmentX(0.5F);
		button1.setText("Stop");
		button1.addActionListener(button1_Click);

		progressBar1 = new JProgressBar();
		progressBar1.setAlignmentX(0.5F);

		label1 = new JLabel();
		label1.setAlignmentX(0.5F);

		JLabel label2 = new JLabel();
		label2.setAlignmentX(0.5F);
		label2.setText(title);

		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		add(label2);
		add(progressBar1);
		add(label1);
		add(button1);
		add(Box.createVerticalStrut(24));

		setName("ProgressView");
		setTitle(title);
		pack();
	}

	public void reportProgress(String stage, int current, int fin) throws StopException
	{
		if (!proceed) 
		{
			throw new StopException();
		}
		label1.setText(stage);
		progressBar1.setMinimum(0);
		progressBar1.setMaximum(fin);
		progressBar1.setValue(current);
	}

	public void reportProgress(int more) throws StopException
	{
		if (!proceed) 
		{
			throw new StopException();
		}
		progressBar1.setValue(progressBar1.getValue() + more);
	}

	public void finished()
	{
		proceed = true;
		try {
			reportProgress("Done", 100, 100);
		} catch (StopException e) {
		}
		if (myplot != null) 
		{
			button1.setText("Plot");
		} 
		else 
		{
			button1.setText("Close");
		}
		finished = true;
		repaint();
	}

	public void setPlot(Plottable plot)
	{
		myplot = plot;
	}

	public PlotView getPlotView()
	{
		if (myplot != null)
		{
			return new PlotView(myplot);
		}
		return null;
	}

	public void closeAll()
	{
		if (myplotview != null) 
		{
			myplotview.setVisible(false);
		}
		setVisible(false);
	}

	private Action button1_Click = new AbstractAction("button1_Click") { public void actionPerformed(ActionEvent e)
	{
		if (!finished)
		{
			proceed = false;
		}
		else if (myplot != null && myplotview == null) 
		{
			myplotview = new PlotView(myplot);
			myplotview.setVisible(true);
			button1.setText("Close");
			repaint();
		}
		else
		{
			closeAll();
		}
	}};
}
