/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.decode;
import java.io.*;
import org.avparade.xml.*;
import org.w3c.dom.*;

/// <summary>
/// Represents an artifact. Limited representation for a7tview.
/// </summary>
public class Artifact
{
	private String id;
	private String description;

	public String getId() { return id; }
	public String getDescription() { return description; }

	public Artifact(String filename) throws Exception
	{
		XMLHandler handler = XMLHandler.Factory.getInstance();
		String fid = filename.toLowerCase();
		int i = fid.lastIndexOf("/");
		if (i < 0) 
		{
			i = fid.lastIndexOf("\\");
		}
		if (i > 0) 
		{
			fid = fid.substring(i);
		}

		Document doc = handler.parse(new File(filename), null);

		NodeList files = doc.getDocumentElement().getElementsByTagName("artifact");
		for (int j = 0; j < files.getLength(); j++) {
			{
				Element a = (Element)files.item(j);
				String tid = a.getAttribute("id");
				if (tid != null && tid.endsWith(fid)) 
				{
					id = tid;
					description = a.getAttribute("description");
					if (description == null) 
					{
						description = "";
					}
					return;
				}
			}
			throw new Exception("Artifact not found");
		}
	}
}

