/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.decode;

    public class Decode
    {
    	private static class Progress implements ProgressWatcher
    	{

			public void closeAll() {
			}

			public void finished() {
				System.err.println();
			}

			public void reportProgress(int more) throws StopException {
			}

			public void reportProgress(String stage, int current, int fin) throws StopException {
				System.err.print(stage + "\r");
			}

			public void setPlot(Plottable plot) {
			}
    		
    	}

        /// <summary>
        /// For console decoding.
        /// </summary>
        public static void main(String[] args) 
        {
        	Progress progress = new Progress();
        	
            for (int i = 0; i < args.length; i++) 
            {
                try
                {
                    TapeDecoder td = new TapeDecoder(args[i]);
                    String front = args[i] + ": ";
                    System.out.println(front + "Start");

                    Source src = td.getSource();
                    int nchannel = src.getNumChannels();
                    TapeParameters tp = td.getTapeParameters();
                    int all = 0;
                    for (; all < nchannel; all++) 
                    {
                        if (tp.getSkew()[all] != 0) 
                        {
                            break;
                        }
                    }
                    if (all == nchannel) 
                    {
                        td.calibrate(progress);
                    }

                    String s = front + "      ";
                    for (int ch = 0; ch < nchannel; ch++) 
                    {
                        s += "  " + src.getChannelName(ch);
                    }
                    System.out.println(s + "  Width " + tp.getWidth() + "  Gather " + tp.getGather());

                    s = front + " Skew ";
                    for (int ch = 0; ch < nchannel; ch++) 
                    {
                        s += "  " + tp.getSkew()[ch];
                    }
                    System.out.println(s + "  Crowd " + tp.getCrowd());

                    s = front + " Clip ";
                    for (int ch = 0; ch < nchannel; ch++) 
                    {
                        s += "  " + tp.getClip(true, ch);
                    }
                    System.out.println(s);

                    int wasrec = td.getNumRecords();
                    td.write(progress, System.out);
                    if (wasrec == 0) 
                    {
                        s = td.getGapJunk() + " Gap Junk";
                    } 
                    else 
                    {
                        s = "";
                    }
                    System.out.println(front + td.getNumRecords() + " Records, " + s + td.getFixupTotal() + " Fixups " + td.getErrorTotal() + " Errors");

                    int[] spacing = td.getSpacingHist();
                    for (int sp = 0; sp < spacing.length; sp++) 
                    {
                        if (spacing[sp] != 0) 
                        {
                        	System.out.println(front + " Spacing " + sp + ": " + spacing[sp] + " Records");
                        }
                    }
                }
                catch (Exception e)
                {
                	e.printStackTrace();
                }
            }
        }
    }
