/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.decode;


    /// <summary>
    /// Summary description for MultiSource.
    /// </summary>
    public interface MultiSource
    {
        DataFile getDataFile();
        Source getSource();
        void calibrate(ProgressWatcher p) throws Exception;
        void calibrate(ProgressWatcher p, MultiSource transfer) throws Exception;
        void mark(ProgressWatcher p) throws Exception;
        void mark(ProgressWatcher p, MultiSource transfer) throws Exception;
        void evaluate(ProgressWatcher p) throws Exception;
        void evaluate(ProgressWatcher p, MultiSource transfer) throws Exception;
        void write(ProgressWatcher p) throws Exception;
        void write(ProgressWatcher p, MultiSource transfer) throws Exception;
        void writeInfo() throws Exception;
        MultiSource clone();
        void close();

        long getStart();
        long getEnd();
        void setStart(long pos);
        void setEnd(long pos);
        long getLength();
        void setPosition(long position);
        void setPosition(long position, long length);
        long getPosition();
        void advance();
        void reEvaluate();
        int getErrorCount();
        int getErrorTotal();
        long getNextError();

        boolean recordOriented();
        int getNumRecords();
        Record getRecord(int index);
        Record getNextRecord();
        Record getNextRecord(long position);
        Record getNextRecord(Record record);
        Record getRecord();
        Record getRecord(long position);
        Record getPreviousRecord();
        Record getPreviousRecord(long position);
        Record getPreviousRecord(Record record);

        boolean hasActivity();
        int getActivityCountHere();
        int getActivityLumpedHere();
        int getActivityStatusHere();
        boolean hasData();
        boolean isDataValidHere();
        int getDataHere();
    }
